﻿using System;
using System.Web;
using System.Text;
using System.Reflection;
using System.Web.UI;
using System.Globalization;
using System.Data;
using System.Collections;
/**********************************************************
 * 项目名称：AjaxLite
 * 开发时间：2007-9-13
 * 开发人员：网魂小兵
 * 联系方式：http://xdotnet.cnblogs.com
 * MSN:myxbing#hotmail.com
 * QQ:30725375
 * 描述：此项目主要用于简单的Ajax回调使用，通过Register方法在Page_Load或之前的周期
 *     中注册回调脚本。此项目的原型来自于CommunityServer,欢迎交流使用。
 * *********************************************************/
namespace AjaxLite
{
    /// <summary>
    /// AJAX方法标记属性，如果标记此方法将可用于AJAX回调。
    /// </summary>
    [AttributeUsage(AttributeTargets.Method)]
    public class AjaxMethodAttribute:Attribute
    {
        private string clientName;
        /// <summary>
        /// 获取或设置客户端的方法。
        /// </summary>
        public string ClientName
        {
            get
            {
                return clientName;
            }
            set
            {
                clientName = value;
            }
        }
    }
    /// <summary>
    /// AJAX方法类。
    /// </summary>
    public class Ajax
    {
        /// <summary>
        /// 从当前请求中获取方法参数对象。
        /// </summary>
        /// <param name="info"></param>
        /// <param name="request"></param>
        /// <returns></returns>
        private static object[] GetParameters(MethodInfo info, HttpRequest request)
        {
            object[] parameters = new object[info.GetParameters().Length];
            int index = 0;
            foreach (ParameterInfo pInfo in info.GetParameters())
            {
                object parameter = null;
                string args = request.Form["AjaxArgs" + index];
                if (args != null)
                {
                    if (pInfo.ParameterType.IsArray)
                    {
                        Type elementType = pInfo.ParameterType.GetElementType();
                        string[] p = args.Split(new char[] { ',' });
                        Array array = Array.CreateInstance(elementType, p.Length);
                        for (int i = 0; i < p.Length; i++)
                        {
                            array.SetValue(Convert.ChangeType(p[i], elementType), i);
                        }
                        parameter = array;
                    }
                    else
                    {
                        parameter = Convert.ChangeType(args, pInfo.ParameterType);
                    }
                }
                parameters[index] = parameter;
                index++;
            }
            return parameters;
        }
        /// <summary>
        /// 查找回调方法。
        /// </summary>
        /// <param name="control"></param>
        /// <returns></returns>
        private static MethodInfo FindMethod(Control control)
        {
            Type type = control.GetType();
            MethodInfo method = type.GetMethod(CallBackMethod);
            if ((method != null) && (method.GetCustomAttributes(typeof(AjaxMethodAttribute), true).Length > 0))
            {
                return method;
            }
            return null;
        }
        /// <summary>
        /// 实例化方法。
        /// </summary>
        /// <param name="control"></param>
        /// <param name="methodInfo"></param>
        /// <param name="parameters"></param>
        /// <returns></returns>
        private static object InvokeMethod(Control control, MethodInfo methodInfo, object[] parameters)
        {
            object method = null;
            try
            {
                method = methodInfo.Invoke(control, parameters);
            }
            catch (TargetInvocationException exception)
            {
                if (exception.InnerException != null)
                {
                    throw exception.InnerException;
                }
                throw exception;
            }
            return method;
        }

        ///// <summary>
        ///// 在登录页面或其他有ReturnURL页面中使用。
        ///// </summary>
        ///// <param name="sender"></param>
        ///// <param name="e"></param>
        //private static void OnReturnURLPreRender(object sender, EventArgs e)
        //{
        //    HttpContext current = HttpContext.Current;
        //    string s = current.Request.QueryString["ReturnURL"];
        //    if ((s != null) && (s.Length > 0))
        //    {
        //        s = current.Server.UrlDecode(s);
        //        if (s.EndsWith("?AjaxCallBack=true", StringComparison.CurrentCultureIgnoreCase) || s.EndsWith("&AjaxCallBack=true", StringComparison.CurrentCultureIgnoreCase))
        //        {
        //            HttpResponse resp = current.Response;
        //            WriteResult(resp, null, "LOGIN");
        //            resp.End();
        //        }
        //    }
        //}

        private static void OnPreRender(object s, EventArgs e)
        {
            if (IsCallBack)
            {
                Control control = s as Control;
                MethodInfo methodInfo = null;
                if (((control != null) && ((control.GetType().FullName == CallBackType) || (control is Page))) && ((control is Page) || (control.ClientID == CallBackID)))
                {
                    methodInfo = FindMethod(control);
                }
                object value = null;
                string error = null;
                HttpResponse resp = HttpContext.Current.Response;
                if (methodInfo != null)
                {
                    try
                    {
                        object[] parameters = GetParameters(methodInfo, HttpContext.Current.Request);
                        value = InvokeMethod(control, methodInfo, parameters);
                    }
                    catch (Exception exception)
                    {
                        error = exception.Message;
                    }
                    WriteResult(resp, value, error);
                    resp.End();
                }
            }
        }
        /// <summary>
        /// 注册Ajax脚本函数，注册方法由命名空间逐级注册。
        /// </summary>
        /// <param name="control">注册脚本所在的控件类型。</param>
        public static void Register(Control control)
        {
            Register(control, control.GetType().FullName);
        }
        /// <summary>
        /// 注册Ajax脚本。
        /// </summary>
        /// <param name="control">注册脚本所在的控件类型。</param>
        /// <param name="prefix">脚本类的名称。</param>
        public static void Register(Control control, string prefix)
        {
            Register(control, prefix, true);
        }
        /// <summary>
        /// 注册页面的Ajax脚本，脚本类名为Ajax。
        /// </summary>
        /// <param name="page">当前页面类型。</param>
        public static void Register(Page page)
        {
            Register(page, "Ajax");
        }
        /// <summary>
        /// 注册页面的Ajax脚本。
        /// </summary>
        /// <param name="page">当前页面类型。</param>
        /// <param name="prefix">注册脚本类名。</param>
        public static void Register(Page page, string prefix)
        {
            Register(page, prefix, false);
        }
        /// <summary>
        /// 注册Ajax脚本。
        /// </summary>
        /// <param name="control">当前控件类型。</param>
        /// <param name="prefix">脚本类型。</param>
        /// <param name="requireID">是否在回调时返回控件ID,此控件一般为用户控件，用于识别回调函数以提高效率。</param>
        public static void Register(Control control, string prefix, bool requireID)
        {
            HttpContext current = HttpContext.Current;
            Type type = control.GetType();
            StringBuilder builder = new StringBuilder();
            builder.Append("<script type=\"text/javascript\">\n// <![CDATA[\n");
            string[] strArray = prefix.Split(new char[] { '.', '+' });
            builder.AppendFormat("var {0} = {{\n", strArray[0]);
            for (int i = 1; i < strArray.Length; i++)
            {
                builder.AppendFormat("\"{0}\": {{\n", strArray[i]);
            }
            int methods = 0;
            foreach (MethodInfo info in type.GetMethods(BindingFlags.Public | BindingFlags.Instance))
            {
                object[] customAttributes = info.GetCustomAttributes(typeof(AjaxMethodAttribute), true);
                if ((customAttributes != null) && (customAttributes.Length > 0))
                {
                    AjaxMethodAttribute attribute = customAttributes[0] as AjaxMethodAttribute;
                    string str = (attribute.ClientName != null) ? attribute.ClientName : info.Name;
                    methods++;
                    builder.AppendFormat("\"{0}\":function(", str);
                    if (requireID)
                    {
                        builder.AppendFormat("id,", new object[0]);
                    }
                    int length = info.GetParameters().Length;
                    for (int k = 0; k < length; k++)
                    {
                        builder.Append(info.GetParameters()[k].Name);
                        //builder.Append("p");
                        //builder.Append(k.ToString());
                        builder.Append(",");
                    }
                    builder.AppendFormat("cb){{\nreturn AjaxCallBack('{0}',{1},'{2}',[", type.FullName, requireID ? "id" : "null", info.Name);
                    for (int m = 0; m < length; m++)
                    {
                        if (m > 0)
                        {
                            builder.Append(",");
                        }
                        builder.Append(info.GetParameters()[m].Name);
                        //builder.Append("p");
                        //builder.Append(m.ToString());
                    }
                    builder.Append("],cb);\n}");
                    builder.Append(",\n");
                }
            }
            if (methods == 0)
            {
                return;
            }
            builder.Length -= 2;
            for (int j = 0; j < strArray.Length; j++)
            {
                builder.Append("}");
            }
            builder.Append(";\n// ]]>\n</script>");
            control.Page.ClientScript.RegisterClientScriptResource(typeof(Ajax), "AjaxLite.Ajax.js");
            control.Page.ClientScript.RegisterClientScriptBlock(control.Page.GetType(), "Ajax:" + prefix, builder.ToString());
            control.PreRender += new EventHandler(Ajax.OnPreRender);
        }

        //public static void RegisterReturnURLPage(Page page)
        //{
        //    page.PreRender += new EventHandler(Ajax.OnReturnURLPreRender);
        //}

        private static void WriteDataRow(StringBuilder sb, DataRow row)
        {
            sb.Append("{");
            foreach (DataColumn column in row.Table.Columns)
            {
                sb.AppendFormat("\"{0}\":", column.ColumnName);
                WriteValue(sb, row[column]);
                sb.Append(",");
            }
            if (row.Table.Columns.Count > 0)
            {
                sb.Length--;
            }
            sb.Append("}");
        }

        private static void WriteDataSet(StringBuilder sb, DataSet ds)
        {
            sb.Append("{\"Tables\":{");
            foreach (DataTable table in ds.Tables)
            {
                sb.AppendFormat("\"{0}\":", table.TableName);
                WriteDataTable(sb, table);
                sb.Append(",");
            }
            if (ds.Tables.Count > 0)
            {
                sb.Length--;
            }
            sb.Append("}}");
        }

        private static void WriteDataTable(StringBuilder sb, DataTable table)
        {
            sb.Append("{\"Rows\":[");
            foreach (DataRow row in table.Rows)
            {
                WriteDataRow(sb, row);
                sb.Append(",");
            }
            if (table.Rows.Count > 0)
            {
                sb.Length--;
            }
            sb.Append("]}");
        }

        private static void WriteEnumerable(StringBuilder sb, IEnumerable e)
        {
            bool flag = false;
            sb.Append("[");
            foreach (object obj2 in e)
            {
                WriteValue(sb, obj2);
                sb.Append(",");
                flag = true;
            }
            if (flag)
            {
                sb.Length--;
            }
            sb.Append("]");
        }

        private static void WriteResult(HttpResponse resp, object value, string error)
        {
            resp.ContentType = "application/x-javascript";
            resp.Cache.SetCacheability(HttpCacheability.NoCache);
            StringBuilder sb = new StringBuilder();
            try
            {
                WriteValueAndError(sb, value, error);
            }
            catch (Exception exception)
            {
                sb.Length = 0;
                WriteValueAndError(sb, null, exception.Message);
            }
            resp.Write(sb.ToString());
        }

        private static void WriteSerializable(StringBuilder sb, object o)
        {
            MemberInfo[] members = o.GetType().GetMembers(BindingFlags.Public | BindingFlags.Instance);
            sb.Append("{");
            bool flag = false;
            foreach (MemberInfo info in members)
            {
                bool isIcluded = false;
                object value = null;
                if ((info.MemberType & MemberTypes.Field) == MemberTypes.Field)
                {
                    value = ((FieldInfo)info).GetValue(o);
                    isIcluded = true;
                }
                else if ((info.MemberType & MemberTypes.Property) == MemberTypes.Property)
                {
                    PropertyInfo info3 = (PropertyInfo)info;
                    if (info3.CanRead && (info3.GetIndexParameters().Length == 0))
                    {
                        value = info3.GetValue(o, null);
                        isIcluded = true;
                    }
                }
                if (isIcluded)
                {
                    sb.Append("\"");
                    sb.Append(info.Name);
                    sb.Append("\":");
                    WriteValue(sb, value);
                    sb.Append(",");
                    flag = true;
                }
            }
            if (flag)
            {
                sb.Length--;
            }
            sb.Append("}");
        }

        private static void WriteString(StringBuilder sb, string s)
        {
            sb.Append("\"");
            foreach (char ch in s)
            {
                switch (ch)
                {
                    case '\b':
                        sb.Append(@"\b");
                        break;

                    case '\t':
                        sb.Append(@"\t");
                        break;

                    case '\n':
                        sb.Append(@"\n");
                        break;

                    case '\f':
                        sb.Append(@"\f");
                        break;

                    case '\r':
                        sb.Append(@"\r");
                        break;

                    case '"':
                        sb.Append("\\\"");
                        break;

                    case '\\':
                        sb.Append(@"\\");
                        break;

                    default:
                        {
                            int num = ch;
                            if ((num < 0x20) || (num > 0x7f))
                            {
                                sb.AppendFormat(@"\u{0:X04}", num);
                            }
                            else
                            {
                                sb.Append(ch);
                            }
                            break;
                        }
                }
            }
            sb.Append("\"");
        }

        private static void WriteValue(StringBuilder sb, object val)
        {
            if ((val == null) || (val == DBNull.Value))
            {
                sb.Append("null");
            }
            else if ((val is string) || (val is Guid))
            {
                WriteString(sb, val.ToString());
            }
            else if (val is bool)
            {
                sb.Append(val.ToString().ToLower());
            }
            else if ((((val is double) || (val is float)) || ((val is long) || (val is int))) || (((val is short) || (val is byte)) || (val is decimal)))
            {
                sb.Append(val);
            }
            else if (val is DateTime)
            {
                sb.Append("new Date(\"");
                sb.Append(((DateTime)val).ToString("MMMM, d yyyy HH:mm:ss", new CultureInfo("zh-CN", false).DateTimeFormat));
                sb.Append("\")");
            }
            else if (val is DataSet)
            {
                WriteDataSet(sb, val as DataSet);
            }
            else if (val is DataTable)
            {
                WriteDataTable(sb, val as DataTable);
            }
            else if (val is DataRow)
            {
                WriteDataRow(sb, val as DataRow);
            }
            else if (val is IEnumerable)
            {
                WriteEnumerable(sb, val as IEnumerable);
            }
            else
            {
                WriteSerializable(sb, val);
            }
        }

        private static void WriteValueAndError(StringBuilder sb, object value, string error)
        {
            sb.Append("{\"value\":");
            WriteValue(sb, value);
            sb.Append(",\"error\":");
            WriteValue(sb, error);
            sb.Append("}");
        }

        /// <summary>
        /// 获取客户端返回控件的ID。
        /// </summary>
        public static string CallBackID
        {
            get
            {
                return HttpContext.Current.Request.Form["AjaxID"];
            }
        }
        /// <summary>
        /// 返回回调函数。
        /// </summary>
        public static string CallBackMethod
        {
            get
            {
                return HttpContext.Current.Request.Form["AjaxMethod"];
            }
        }
        /// <summary>
        /// 返回回调函数所在的类型，一般是控件所在页面的类型。
        /// </summary>
        public static string CallBackType
        {
            get
            {
                return HttpContext.Current.Request.Form["AjaxType"];
            }
        }
        /// <summary>
        /// 返回当前请求是否是Ajax回调。
        /// </summary>
        public static bool IsCallBack
        {
            get
            {
                return (CallBackType != null);
            }
        }

    }
}
